@@ -9,6 +9,7 @@ Shell Widget for the IPython Console
 """
 
 # Standard library imports
+import logging
 import os
 import os.path as osp
 import time
@@ -45,8 +46,11 @@ from spyder.utils.clipboard_helper import CLIPBOARD_HELPER
 from spyder.widgets.helperwidgets import MessageCheckBox
 
 
+logger = logging.getLogger(__name__)
+
 MODULES_FAQ_URL = (
-    "https://docs.spyder-ide.org/5/faq.html#using-packages-installer")
+    "https://docs.spyder-ide.org/5/faq.html#using-packages-installer"
+)
 
 
 class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
@@ -168,6 +172,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
 
         # Keyboard shortcuts
         # Registered here to use shellwidget as the parent
+        SpyderWidgetMixin.__init__(self)
         self.regiter_shortcuts()
 
         # Set the color of the matched parentheses here since the qtconsole
@@ -182,6 +187,11 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         self._kernel_configuration = {}
         self.is_kernel_configured = False
         self._init_kernel_setup = False
+        self._is_banner_shown = False
+
+        # Set bright colors instead of bold formatting for better traceback
+        # readability.
+        self._ansi_processor.bold_text_enabled = False
 
         if handlers is None:
             handlers = {}
@@ -199,9 +209,6 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         self._execute_queue = []
         self.executed.connect(self.pop_execute_queue)
 
-        # To show the console banner on first prompt
-        self.sig_prompt_ready.connect(self._show_banner)
-
         # Show a message in our installers to explain users how to use
         # modules that don't come with them.
         self.show_modules_message = is_conda_based_app()
@@ -317,6 +324,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
             KernelConnectionState.SpyderKernelReady
         ):
             self.setup_spyder_kernel()
+            self._show_banner()
 
     def handle_kernel_connection_error(self):
         """An error occurred when connecting to the kernel."""
@@ -694,7 +702,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         self.style_sheet, dark_color = create_qss_style(color_scheme)
         self.syntax_style = color_scheme
         self._style_sheet_changed()
-        self._syntax_style_changed()
+        self._syntax_style_changed(changed={})
         if reset:
             self.reset(clear=True)
         if not self.spyder_kernel_ready:
@@ -705,10 +713,14 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         )
 
     def update_syspath(self, path_dict, new_path_dict):
-        """Update sys.path contents on kernel."""
-        self.call_kernel(
-            interrupt=True,
-            blocking=False).update_syspath(path_dict, new_path_dict)
+        """Update sys.path contents in the kernel."""
+        # Prevent error when the kernel is not available and users open/close
+        # projects or use the Python path manager.
+        # Fixes spyder-ide/spyder#21563
+        if self.kernel_handler is not None:
+            self.call_kernel(interrupt=True, blocking=False).update_syspath(
+                path_dict, new_path_dict
+            )
 
     def request_syspath(self):
         """Ask the kernel for sys.path contents."""
@@ -792,7 +804,9 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
             )
 
             banner = ''.join(banner_parts)
-        except CommError:
+        except (CommError, TimeoutError, RuntimeError):
+            # RuntimeError happens when the kernel crashes after it starts.
+            # See spyder-ide/spyder#22929
             banner = ""
 
         # Pylab additions
@@ -830,8 +844,8 @@ overrided by the Sympy module (e.g. plot)
             env_info = self.get_pythonenv_info()
             py_ver = env_info['python_version']
             ipy_ver = env_info['ipython_version']
-            banner = f'Python {py_ver} -- IPython {ipy_ver}'
-        except CommError:
+            banner = f'Python {py_ver} -- IPython {ipy_ver}\n'
+        except (CommError, TimeoutError):
             banner = ""
 
         return banner
@@ -1178,11 +1192,17 @@ overrided by the Sympy module (e.g. plot)
 
     def _show_banner(self):
         """Show banner before first prompt."""
-        # Don't show banner for external kernels
-        if self.is_external_kernel and not self.is_remote():
-            return ""
+        if (
+            # Don't show banner for external but local kernels
+            self.is_external_kernel and not self.is_remote()
+            # Don't show it if it was already shown
+            or self._is_banner_shown
+        ):
+            return
 
-        # Detect what kind of banner we want to show
+        logger.debug(f"Showing banner for {self}")
+
+        # Check what kind of banner we want to show
         show_banner_o = self.additional_options['show_banner']
         if show_banner_o:
             banner = self.long_banner()
@@ -1194,8 +1214,13 @@ overrided by the Sympy module (e.g. plot)
         cursor.setPosition(0)
         self._insert_plain_text(cursor, banner)
 
+        # We need to do this so the banner is available to other QtConsole
+        # methods (e.g. console resets).
+        # Fixes spyder-ide/spyder#22593
+        self.banner = banner
+
         # Only do this once
-        self.sig_prompt_ready.disconnect(self._show_banner)
+        self._is_banner_shown = True
 
     # ---- Private API (overrode by us)
     def _event_filter_console_keypress(self, event):
@@ -1343,6 +1368,12 @@ overrided by the Sympy module (e.g. plot)
                     section=ClientContextMenuSections.Clear
                 )
 
+            self.add_item_to_menu(
+                self.get_action(ClientContextMenuActions.Quit),
+                context_menu,
+                section=ClientContextMenuSections.Quit
+            )
+
         return context_menu
 
     def _banner_default(self):
@@ -1377,10 +1408,14 @@ overrided by the Sympy module (e.g. plot)
             and not self.is_remote()
         ):
             # The kernel might never restart, show position of fault file
-            msg += (
-                "\n" + _("Its crash file is located at:") + " "
-                + self.kernel_handler.fault_filename()
-            )
+            # if available else show kernel error
+            if self.kernel_handler.fault_filename():
+                msg += (
+                    "\n" + _("Its crash file is located at:") + " "
+                    + self.kernel_handler.fault_filename()
+                )
+            else:
+                self.ipyclient.show_kernel_connection_error()
 
         self._append_html(f"<br>{msg}<br>", before_prompt=False)
         self.insert_horizontal_ruler()
@@ -1407,6 +1442,12 @@ overrided by the Sympy module (e.g. plot)
         if self.syntax_style:
             self._highlighter._style = create_style_class(self.syntax_style)
             self._highlighter._clear_caches()
+            if changed is None:
+                return
+            self.set_kernel_configuration(
+                "traceback_highlight_style",
+                get_color_scheme(self.syntax_style),
+            )
         else:
             self._highlighter.set_style_sheet(self.style_sheet)
 
@@ -1461,3 +1502,11 @@ overrided by the Sympy module (e.g. plot)
         """Reimplement Qt method to send focus change notification"""
         self.sig_focus_changed.emit()
         return super(ShellWidget, self).focusOutEvent(event)
+
+    # ---- Python methods
+    def __repr__(self):
+        # Handy repr for logging.
+        # Solution from https://stackoverflow.com/a/121508/438386
+        return (
+            "<" + self.__class__.__name__ + " object at " + hex(id(self)) + ">"
+        )
